import tornado.web
from models import User, Session
import bcrypt
from utils import validate_input, set_karma

class RegisterHandler(tornado.web.RequestHandler):
    async def get(self):
        self.render("register.html", username=None)

    async def post(self):
        day = await validate_input(self.get_arguments("day"), int, "1-31")
        if not day: 
            self.render("error.html", type="error", message="Invalid day format")
            return
        
        month = await validate_input(self.get_arguments("month"), int, "1-12")
        if not month: 
            self.render("error.html", type="error", message="Invalid month format")
            return
        
        year = await validate_input(self.get_arguments("year"), int, "1800-2024")
        if not year: 
            self.render("error.html", type="error", message="Invalid year format")
            return

        firstname = await validate_input(self.get_arguments("firstname"), str)
        lastname = await validate_input(self.get_arguments("lastname"), str)
        username = await validate_input(self.get_arguments("username"), str)
        password = await validate_input(self.get_arguments("password"), str)
        caste = await validate_input(self.get_arguments("caste"), str)
        if not username or not password or not caste or not firstname or not lastname:
            self.render("error.html", type="error", message="Only A-z charset allowed for string parameters")
            return
        
        if len(password) < 10 or len(username) < 10:
            self.render("error.html", type="error", message="Username and password should be no less than 10 charecters")
            return
        
        if caste not in ["Shudras","Vaishyas","Kshatriyas","Brahmins"]:
            self.render("error.html", type="error", message="Unknown caste {}".format(caste))
            return
        
        hashed_password = bcrypt.hashpw(password.encode('utf-8'), bcrypt.gensalt())
        session = Session()
        user = session.query(User).filter(User.username == username).first()
        if user:
            self.render("error.html", type="error", message="Username is already taken")
            return
        
        new_user = User(username=username, password=hashed_password.decode('utf-8'), caste=caste, firstname=firstname, lastname=lastname, day=day, month=month, year=year)
        session.add(new_user)
        session.commit()
        karma = await set_karma(day, month, year, firstname, lastname)
        if karma:
            user_to_update = session.query(User).filter(User.username == username).first()
            if user_to_update:
                user_to_update.karma = user_to_update.karma-int(karma)
                session.commit()
                session.close()
                self.redirect("/login")
        else:
            self.render("error.html", type="error", message="Setting karma failed")
            session.close()
        return


class LoginHandler(tornado.web.RequestHandler):
    async def get(self):
        self.render("login.html", username=None)

    async def post(self):        
        username = await validate_input(self.get_arguments("username"), str)
        password = await validate_input(self.get_arguments("password"), str)
        if not username or not password:
            self.render("error.html", type="error", message="Only A-z charset allowed for string parameters")
            return
        
        password = password.encode('utf-8')
        user = Session().query(User).filter(User.username == username).first()
        if user:
            if user.karma < 0:
                self.render("error.html", type="error", message="Sinners are not allowed to our portal")
                return
            stored_password = user.password
            if isinstance(stored_password, str):
                stored_password = stored_password.encode('utf-8')
            if bcrypt.checkpw(password, stored_password):
                if user.totp_enabled == True:
                    self.set_secure_cookie("tmp_user", str(user.id))
                    self.redirect("/2fa")
                else:
                    self.set_secure_cookie("user", str(user.id))
                    self.redirect("/role")
            else:
                self.render("error.html", type="error", message="Login failed")
        else:
            self.render("error.html", type="error", message="User not found")
        return


class Role(tornado.web.RequestHandler):
    async def get(self):
        current_role = self.get_secure_cookie("role")
        user_id = self.get_secure_cookie("user")
        if not user_id:
            self.redirect("/login")
            return

        if current_role:
            self.redirect("/account")
        self.render("role.html", username=None)

    async def post(self):
        current_role = self.get_secure_cookie("role")
        user_id = self.get_secure_cookie("user")
        if not user_id:
            self.redirect("/login")
            return
        if current_role:
            self.redirect("/account")
        new_role = await validate_input(self.get_arguments("role"))
        if not new_role:
            self.render("error.html", type="error", message="Role was not provided or had an invalid format")
        if new_role in ["guru", "rishi"]:
            self.set_secure_cookie("role", new_role)
            self.redirect("/account")
        else:
            self.render("error.html", type="error", message="Role {} does not exist".format(new_role))
            self.redirect("/role")


class Logout(tornado.web.RequestHandler):
    async def get(self):
        user_id = self.get_secure_cookie("user")
        if user_id:
            self.clear_cookie("user")
        current_role = self.get_secure_cookie("role")
        if current_role:
            self.clear_cookie("role")
        tmp_user = self.get_secure_cookie("tmp_user")
        if tmp_user:
            self.clear_cookie("tmp_user")
        self.redirect("/login")

